# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"FitTask": "00_base.ipynb",
         "BaseITEEstimator": "00_base.ipynb",
         "BaseActiveLearner": "00_base.ipynb",
         "BaseAcquisitionFunction": "00_base.ipynb",
         "BaseAssignmentFunction": "00_base.ipynb",
         "BaseStoppingRule": "00_base.ipynb",
         "BaseDataGenerator": "00_base.ipynb",
         "RandomAcquisitionFunction": "01_models.ipynb",
         "UncertaintyAcquisitionFunction": "01_models.ipynb",
         "TypeSAcquistionFunction": "01_models.ipynb",
         "EMCMAcquisitionFunction": "01_models.ipynb",
         "RandomAssignmentFunction": "01_models.ipynb",
         "UncertaintyAssignmentFunction": "01_models.ipynb",
         "MajorityAssignmentFunction": "01_models.ipynb",
         "PMajorityAssignmentFunction": "01_models.ipynb",
         "CausalForestEstimator": "02_estimators.ipynb",
         "OPENBTITEEstimator": "02_estimators.ipynb",
         "GPEstimator": "02_estimators.ipynb",
         "get_ihdp_dict": "99_misc.ipynb"}

modules = ["base.py",
           "models.py",
           "estimators.py",
           "helper.py"]

doc_url = "https://puhazoli.github.io/asbe/"

git_url = "https://github.com/puhazoli/asbe/tree/master/"

def custom_doc_links(name): return None
